clear * 

load sources

J = 3; % the number of sources (max = 4)
I = J; % the number of microphones

N = length(s); % the number of samples

% Parameters of the RIR generator by E. Habets, download at 
% https://www.audiolabs-erlangen.de/fau/professor/habets/software/rir-generator
c = 340;                     % Sound velocity (m/s)
room = [ 5 4 3 ];               % Room dimensions [ x y z ] (m)
beta = 0.150;                % Reverberation time (s)
nsample = 4000;             % Number of samples
mtype = 'hypercardioid';     % Type of microphone
order = -1;                  % -1 equals maximum reflection order!
dim = 3;                     % Room dimension
orientation = [-pi/2 0];      % Microphone orientation [azimuth elevation] in radians
hp_filter = 1;               % Enable high-pass filter


% source/microphones positions

r(1,:) = [3    2 1];           % microphone 1 position [ x y z ] (m)
r(2,:) = [3.05 2 1];           % microphone 2 position [ x y z ] (m)
r(3,:) = [3.1  2 1];           % microphone 3 position [ x y z ] (m)
r(4,:) = [3.15 2 1];           % microphone 4 position [ x y z ] (m)

p(1,:) = [2 1 1];           % source 1 position [ x y z ] (m)
p(2,:) = [3 1 1];           % source 1 position [ x y z ] (m)
p(3,:) = [4 1 1];           % source 1 position [ x y z ] (m)
p(4,:) = [4 2 1];           % source 1 position [ x y z ] (m)


% generate RIRs and source images

RIR = zeros(I, nsample, J);
images = zeros(I, N, J);

for j = 1:J
    for i = 1:I
        RIR(i,:,j) = rir_generator(c, fs, r(i,:), p(j,:), room, beta, nsample, ...
            mtype, order, dim, orientation, hp_filter);
        images(i,:,j) = filter(RIR(i,:,j),1,s(:,j));
    end
end

% Mixed signals at microphones

x = sum(images, 3);

% Perform the separation HERE
% ...
% The output should be the separated source images in matrix 
% sep_images (the same dimensions as images)

% For example, using the time-domain T-ABCD method available at
% https://asap.ite.tul.cz/downloads/t-abcd-a-time-domain-method-for-blind-audio-source-separation-based-on-a-complete-ica-decomposition-of-an-observation-space/

L                = 20; % the length of separating filter
opts.lengthofICA = 16000; % the length of data used for ICA
opts.offsetICA   = 1; % offset of data used for ICA
opts.method      = 'bgl'; % ICA method
opts.wthreshold  = 0.8; % threshold for the selection of independent components 0<w<1

[~, sep_images] = bssdeconv(x, L, opts);
% A user-interface application can be executed by: tabcd2(x,fs)


% The evaluation using BSS_Eval
% Download at http://bass-db.gforge.inria.fr/bss_eval/

[SDR, ISR, SIR, SAR, perm] = ...
    bss_eval_images(permute(sep_images,[3 2 1]), permute(images,[3 2 1]));

